/*
 * PIO "in" transfers can cause D-cache lines to be allocated
 * to the data being read. If the target is the page cache then
 * the kernel can create a user space mapping of the same page
 * without flushing it from the D-cache. This has large potential
 * to create cache aliases. The Cobalts seem to trigger this
 * problem easily.
 *
 * MIPs doesn't have a flush_dcache_range() so we roll
 * our own.
 *
 * -- pdh
 */

#include <asm/r4kcache.h>

static inline void __flush_dcache(void)
{
	unsigned long dc_size, dc_line, addr, end;

	dc_size = current_cpu_data.dcache.ways << current_cpu_data.dcache.waybit;
	dc_line = current_cpu_data.dcache.linesz;

	addr = KSEG0;
	end = addr + dc_size;

	for (; addr < end; addr += dc_line)
		flush_dcache_line_indexed(addr);
}

static inline void __flush_dcache_range(unsigned long start, unsigned long end)
{
	unsigned long dc_size, dc_line, addr;

	dc_size = current_cpu_data.dcache.ways << current_cpu_data.dcache.waybit;
	dc_line = current_cpu_data.dcache.linesz;

	addr = start & ~(dc_line - 1);
	end += dc_line - 1;

	if (end - addr < dc_size)
		for (; addr < end; addr += dc_line)
			flush_dcache_line(addr);
	else
		__flush_dcache();
}

static inline void __ide_insw(unsigned long port, void *addr, unsigned int count)
{
	__insw(port, addr, count);

	__flush_dcache_range((unsigned long) addr, (unsigned long) addr + count * 2);
}

static inline void __ide_insl(unsigned long port, void *addr, unsigned int count)
{
	__insl(port, addr, count);

	__flush_dcache_range((unsigned long) addr, (unsigned long) addr + count * 4);
}

#undef insw
#undef insl

#define insw(p,a,n)		__ide_insw((p),(a),(n))
#define insl(p,a,n)		__ide_insl((p),(a),(n))

#define __ide_mm_insw(p,a,n)	do{BUG();}while(0)
#define __ide_mm_insl(p,a,n)	do{BUG();}while(0)
#define __ide_mm_outsw(p,a,n)	do{BUG();}while(0)
#define __ide_mm_outsl(p,a,n)	do{BUG();}while(0)
